@@ -15,7 +15,6 @@
 #    under the License.
 
 import functools
-import six
 
 import nova.api.openstack
 from nova.api.openstack.compute import admin_actions
@@ -454,6 +453,9 @@ ROUTE_LIST = (
     ('/os-aggregates/{id}/action', {
         'POST': [aggregates_controller, 'action'],
     }),
+    ('/os-aggregates/{id}/images', {
+        'POST': [aggregates_controller, 'images'],
+    }),
     ('/os-assisted-volume-snapshots', {
         'POST': [assisted_volume_snapshots_controller, 'create']
     }),
@@ -863,7 +865,7 @@ class APIRouterV21(base_wsgi.Router):
             # For example, the request to the '' will be redirect to the '/' in
             # the Nova API. To indicate that, using the target path instead of
             # a dict. The route entry just writes as "('', '/)".
-            if isinstance(methods, six.string_types):
+            if isinstance(methods, str):
                 self.map.redirect(path, methods)
                 continue
 
