@@ -17,7 +17,6 @@ import functools
 
 import netaddr
 from oslo_serialization import jsonutils
-import six
 
 from nova import exception
 from nova.i18n import _
@@ -104,14 +103,16 @@ VNIC_TYPE_MACVTAP = 'macvtap'
 VNIC_TYPE_DIRECT_PHYSICAL = 'direct-physical'
 VNIC_TYPE_BAREMETAL = 'baremetal'
 VNIC_TYPE_VIRTIO_FORWARDER = 'virtio-forwarder'
+VNIC_TYPE_VDPA = 'vdpa'
 
 # Define list of ports which needs pci request.
 # Note: The macvtap port needs a PCI request as it is a tap interface
 # with VF as the lower physical interface.
 # Note: Currently, VNIC_TYPE_VIRTIO_FORWARDER assumes a 1:1
 # relationship with a VF. This is expected to change in the future.
-VNIC_TYPES_SRIOV = (VNIC_TYPE_DIRECT, VNIC_TYPE_MACVTAP,
-                    VNIC_TYPE_DIRECT_PHYSICAL, VNIC_TYPE_VIRTIO_FORWARDER)
+VNIC_TYPES_SRIOV = (
+    VNIC_TYPE_DIRECT, VNIC_TYPE_MACVTAP, VNIC_TYPE_DIRECT_PHYSICAL,
+    VNIC_TYPE_VIRTIO_FORWARDER, VNIC_TYPE_VDPA)
 
 # Define list of ports which are passthrough to the guest
 # and need a special treatment on snapshot and suspend/resume
@@ -469,6 +470,14 @@ class VIF(Model):
         return (self.is_hybrid_plug_enabled() and not
                 migration.is_same_host())
 
+    @property
+    def has_live_migration_plug_time_event(self):
+        """Returns whether this VIF's network-vif-plugged external event will
+        be sent by Neutron at "plugtime" - in other words, as soon as neutron
+        completes configuring the network backend.
+        """
+        return self.is_hybrid_plug_enabled()
+
     def is_hybrid_plug_enabled(self):
         return self['details'].get(VIF_DETAILS_OVS_HYBRID_PLUG, False)
 
@@ -487,6 +496,9 @@ class VIF(Model):
         vif['network'] = Network.hydrate(vif['network'])
         return vif
 
+    def has_allocation(self):
+        return self['profile'] and bool(self['profile'].get('allocation'))
+
 
 def get_netmask(ip, subnet):
     """Returns the netmask appropriate for injection into a guest."""
@@ -510,7 +522,7 @@ class NetworkInfo(list):
 
     @classmethod
     def hydrate(cls, network_info):
-        if isinstance(network_info, six.string_types):
+        if isinstance(network_info, str):
             network_info = jsonutils.loads(network_info)
         return cls([VIF.hydrate(vif) for vif in network_info])
 
@@ -527,19 +539,29 @@ class NetworkInfo(list):
         return jsonutils.dumps(self)
 
     def get_bind_time_events(self, migration):
-        """Returns whether any of our VIFs have "bind-time" events. See
-        has_bind_time_event() docstring for more details.
+        """Returns a list of external events for any VIFs that have
+        "bind-time" events during cold migration.
         """
         return [('network-vif-plugged', vif['id'])
                 for vif in self if vif.has_bind_time_event(migration)]
 
+    def get_live_migration_plug_time_events(self):
+        """Returns a list of external events for any VIFs that have
+        "plug-time" events during live migration.
+        """
+        return [('network-vif-plugged', vif['id'])
+                for vif in self if vif.has_live_migration_plug_time_event]
+
     def get_plug_time_events(self, migration):
-        """Complementary to get_bind_time_events(), any event that does not
-        fall in that category is a plug-time event.
+        """Returns a list of external events for any VIFs that have
+        "plug-time" events during cold migration.
         """
         return [('network-vif-plugged', vif['id'])
                 for vif in self if not vif.has_bind_time_event(migration)]
 
+    def has_port_with_allocation(self):
+        return any(vif.has_allocation() for vif in self)
+
 
 class NetworkInfoAsyncWrapper(NetworkInfo):
     """Wrapper around NetworkInfo that allows retrieving NetworkInfo
