@@ -14,7 +14,6 @@
 #    under the License.
 
 import mock
-import six
 import testtools
 
 from oslo_config import cfg
@@ -60,15 +59,7 @@ class NUMAServersTest(NUMAServersTestBase):
                         filter_called_on_error=True,
                         expected_usage=None):
 
-        # NOTE(bhagyashris): Always use host as 'compute1' so that it's
-        # possible to get resource provider information for verifying
-        # compute usages. This host name 'compute1' is hard coded in
-        # Connection class in fakelibvirt.py.
-        # TODO(stephenfin): Remove the hardcoded limit, possibly overridding
-        # 'start_service' to make sure there isn't a mismatch
-        self.compute = self.start_service('compute', host='compute1')
-
-        compute_rp_uuid = self.placement_api.get(
+        compute_rp_uuid = self.placement.get(
             '/resource_providers?name=compute1').body[
             'resource_providers'][0]['uuid']
 
@@ -78,10 +69,14 @@ class NUMAServersTest(NUMAServersTestBase):
             expected_state=end_status)
 
         # Validate the quota usage
-        if filter_called_on_error and end_status == 'ACTIVE':
+        if (
+            filter_called_on_error and expected_usage and
+            end_status == 'ACTIVE'
+        ):
             quota_details = self.api.get_quota_detail()
-            expected_core_usages = expected_usage.get(
-                'VCPU', expected_usage.get('PCPU', 0))
+            expected_core_usages = (
+                expected_usage.get('VCPU', 0) + expected_usage.get('PCPU', 0)
+            )
             self.assertEqual(expected_core_usages,
                              quota_details['cores']['in_use'])
 
@@ -94,7 +89,7 @@ class NUMAServersTest(NUMAServersTestBase):
             self.assertFalse(self.mock_filter.called)
 
         if expected_usage:
-            compute_usage = self.placement_api.get(
+            compute_usage = self.placement.get(
                 '/resource_providers/%s/usages' % compute_rp_uuid).body[
                     'usages']
             self.assertEqual(expected_usage, compute_usage)
@@ -110,10 +105,8 @@ class NUMAServersTest(NUMAServersTestBase):
         """
 
         host_info = fakelibvirt.HostInfo(cpu_nodes=2, cpu_sockets=1,
-                                         cpu_cores=2, cpu_threads=2,
-                                         kB_mem=15740000)
-        fake_connection = self._get_connection(host_info=host_info)
-        self.mock_conn.return_value = fake_connection
+                                         cpu_cores=2, cpu_threads=2)
+        self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {'hw:numa_nodes': '2'}
         flavor_id = self._create_flavor(vcpu=2, extra_spec=extra_spec)
@@ -134,10 +127,8 @@ class NUMAServersTest(NUMAServersTestBase):
         separate host NUMA node.
         """
 
-        host_info = fakelibvirt.HostInfo(cpu_nodes=1, cpu_sockets=1,
-                                         cpu_cores=2, kB_mem=15740000)
-        fake_connection = self._get_connection(host_info=host_info)
-        self.mock_conn.return_value = fake_connection
+        host_info = fakelibvirt.HostInfo()
+        self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {'hw:numa_nodes': '2'}
         flavor_id = self._create_flavor(extra_spec=extra_spec)
@@ -150,11 +141,9 @@ class NUMAServersTest(NUMAServersTestBase):
         Configuring huge pages against a server also necessitates configuring a
         NUMA topology.
         """
-        host_info = fakelibvirt.HostInfo(cpu_nodes=1, cpu_sockets=2,
-                                         cpu_cores=2, cpu_threads=2,
-                                         kB_mem=(1024 * 1024 * 16))  # GB
-        self.mock_conn.return_value = self._get_connection(host_info=host_info)
 
+        host_info = fakelibvirt.HostInfo(cpu_nodes=1, cpu_sockets=2,
+                                         cpu_cores=2, cpu_threads=2)
         # create 1024 * 2 MB huge pages, and allocate the rest of the 16 GB as
         # small pages
         for cell in host_info.numa_topology.cells:
@@ -164,6 +153,7 @@ class NUMAServersTest(NUMAServersTestBase):
                 (4, small_pages),
                 (2048, huge_pages),
             ])
+        self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {'hw:mem_page_size': 'large'}
         flavor_id = self._create_flavor(memory_mb=2048, extra_spec=extra_spec)
@@ -182,10 +172,10 @@ class NUMAServersTest(NUMAServersTestBase):
 
         This should fail because there are hugepages but not enough of them.
         """
+
         host_info = fakelibvirt.HostInfo(cpu_nodes=1, cpu_sockets=2,
-                                         cpu_cores=2, cpu_threads=2,
-                                         kB_mem=(1024 * 1024 * 16))  # GB
-        self.mock_conn.return_value = self._get_connection(host_info=host_info)
+                                         cpu_cores=2, cpu_threads=2)
+        self.start_compute(host_info=host_info, hostname='compute1')
 
         # create 512 * 2 MB huge pages, and allocate the rest of the 16 GB as
         # small pages
@@ -202,8 +192,8 @@ class NUMAServersTest(NUMAServersTestBase):
 
         self._run_build_test(flavor_id, end_status='ERROR')
 
-    def test_create_server_with_legacy_pinning_policy(self):
-        """Create a server using the legacy 'hw:cpu_policy' extra spec.
+    def test_create_server_with_dedicated_policy(self):
+        """Create a server using the 'hw:cpu_policy=dedicated' extra spec.
 
         This should pass and result in a guest NUMA topology with pinned CPUs.
         """
@@ -213,10 +203,8 @@ class NUMAServersTest(NUMAServersTestBase):
         self.flags(vcpu_pin_set=None)
 
         host_info = fakelibvirt.HostInfo(cpu_nodes=1, cpu_sockets=1,
-                                         cpu_cores=5, cpu_threads=2,
-                                         kB_mem=15740000)
-        fake_connection = self._get_connection(host_info=host_info)
-        self.mock_conn.return_value = fake_connection
+                                         cpu_cores=5, cpu_threads=2)
+        self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {
             'hw:cpu_policy': 'dedicated',
@@ -231,7 +219,106 @@ class NUMAServersTest(NUMAServersTestBase):
         self.assertEqual(1, len(inst.numa_topology.cells))
         self.assertEqual(5, inst.numa_topology.cells[0].cpu_topology.cores)
 
-    def test_create_server_with_legacy_pinning_policy_old_configuration(self):
+    def test_create_server_with_mixed_policy(self):
+        """Create a server using the 'hw:cpu_policy=mixed' extra spec.
+
+        This should pass and result in a guest NUMA topology with a mixture of
+        pinned and unpinned CPUs.
+        """
+
+        # configure the flags so we 2 shared, 2 dedicated CPUs on one node and
+        # 1 shared and 3 dedicated on the other; the guest will request the
+        # latter so it should always land on the second NUMA node
+        self.flags(
+            cpu_dedicated_set='2-3,5-7', cpu_shared_set='0,1,4',
+            group='compute')
+        self.flags(vcpu_pin_set=None)
+
+        host_info = fakelibvirt.HostInfo(
+            cpu_nodes=2, cpu_sockets=1, cpu_cores=4, cpu_threads=1)
+        self.start_compute(host_info=host_info, hostname='compute1')
+
+        # sanity check the created host topology object; this is really just a
+        # test of the fakelibvirt module
+        host_numa = objects.NUMATopology.obj_from_db_obj(
+            objects.ComputeNode.get_by_nodename(
+                self.ctxt, 'compute1',
+            ).numa_topology
+        )
+        self.assertEqual(2, len(host_numa.cells))
+        self.assertEqual({0, 1}, host_numa.cells[0].cpuset)
+        self.assertEqual({2, 3}, host_numa.cells[0].pcpuset)
+        self.assertEqual({4}, host_numa.cells[1].cpuset)
+        self.assertEqual({5, 6, 7}, host_numa.cells[1].pcpuset)
+
+        # create a flavor with 1 shared and 3 dedicated CPUs so that we can
+        # validate that both come from the same host NUMA node
+        extra_spec = {
+            'hw:cpu_policy': 'mixed',
+            'hw:cpu_dedicated_mask': '^0',
+        }
+        flavor_id = self._create_flavor(vcpu=4, extra_spec=extra_spec)
+        expected_usage = {
+            'DISK_GB': 20, 'MEMORY_MB': 2048, 'PCPU': 3, 'VCPU': 1,
+        }
+
+        server = self._run_build_test(flavor_id, expected_usage=expected_usage)
+
+        # sanity check the instance topology
+        inst = objects.Instance.get_by_uuid(self.ctxt, server['id'])
+        self.assertEqual(1, len(inst.numa_topology.cells))
+        self.assertEqual(4, inst.numa_topology.cells[0].cpu_topology.cores)
+        self.assertEqual({0}, inst.numa_topology.cells[0].cpuset)
+        self.assertEqual({1, 2, 3}, inst.numa_topology.cells[0].pcpuset)
+        self.assertEqual(
+            {5, 6, 7}, set(inst.numa_topology.cells[0].cpu_pinning.values()),
+        )
+
+    def test_create_server_with_mixed_policy_fails(self):
+        """Create a server using the 'hw:cpu_policy=mixed' extra spec on a host
+        with insufficient shared cores on one node and dedicated cores on the
+        other.
+
+        This should fail since both shared and dedicated instance cores should
+        come from the same host node.
+        """
+        # configure the flags so we mark all cores on one node as shared and
+        # all cores on the other as dedicated; the guest shouldn't be able to
+        # schedule to this unless using a multi-node topology itself
+        self.flags(
+            cpu_dedicated_set='4-7', cpu_shared_set='0-3',
+            group='compute')
+        self.flags(vcpu_pin_set=None)
+
+        host_info = fakelibvirt.HostInfo(
+            cpu_nodes=2, cpu_sockets=1, cpu_cores=4, cpu_threads=1)
+        self.start_compute(host_info=host_info, hostname='compute1')
+
+        # sanity check the created host topology object; this is really just a
+        # test of the fakelibvirt module
+        host_numa = objects.NUMATopology.obj_from_db_obj(
+            objects.ComputeNode.get_by_nodename(
+                self.ctxt, 'compute1',
+            ).numa_topology
+        )
+        self.assertEqual(2, len(host_numa.cells))
+        self.assertEqual({0, 1, 2, 3}, host_numa.cells[0].cpuset)
+        self.assertEqual(set(), host_numa.cells[0].pcpuset)
+        self.assertEqual(set(), host_numa.cells[1].cpuset)
+        self.assertEqual({4, 5, 6, 7}, host_numa.cells[1].pcpuset)
+
+        # create a flavor with 1 shared and 3 dedicated CPUs so that we can
+        # validate that it isn't schedulable
+        extra_spec = {
+            'hw:cpu_policy': 'mixed',
+            'hw:cpu_dedicated_mask': '^0',
+        }
+        flavor_id = self._create_flavor(vcpu=4, extra_spec=extra_spec)
+
+        # There shouldn't be any hosts available to satisfy this request
+        self._run_build_test(flavor_id, end_status='ERROR')
+
+    def test_create_server_with_dedicated_policy_old_configuration(self):
         """Create a server using the legacy extra spec and configuration.
 
         This should pass and result in a guest NUMA topology with pinned CPUs,
@@ -244,10 +331,8 @@ class NUMAServersTest(NUMAServersTestBase):
         self.flags(vcpu_pin_set='0-7')
 
         host_info = fakelibvirt.HostInfo(cpu_nodes=2, cpu_sockets=1,
-                                         cpu_cores=2, cpu_threads=2,
-                                         kB_mem=15740000)
-        fake_connection = self._get_connection(host_info=host_info)
-        self.mock_conn.return_value = fake_connection
+                                         cpu_cores=2, cpu_threads=2)
+        self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {
             'hw:cpu_policy': 'dedicated',
@@ -274,11 +359,8 @@ class NUMAServersTest(NUMAServersTestBase):
         # host has hyperthreads, which means we're going to end up consuming
         # $flavor.vcpu hosts cores plus the thread sibling(s) for each core
         host_info = fakelibvirt.HostInfo(
-            cpu_nodes=1, cpu_sockets=1, cpu_cores=2, cpu_threads=2,
-            kB_mem=(1024 * 1024 * 16),  # GB
-        )
-        fake_connection = self._get_connection(host_info=host_info)
-        self.mock_conn.return_value = fake_connection
+            cpu_nodes=1, cpu_sockets=1, cpu_cores=2, cpu_threads=2)
+        self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {
             'hw:cpu_policy': 'dedicated',
@@ -298,7 +380,7 @@ class NUMAServersTest(NUMAServersTestBase):
         )
         self.assertEqual({0, 1, 2, 3}, host_numa.cells[0].pinned_cpus)
 
-    def test_create_server_with_legacy_pinning_policy_fails(self):
+    def test_create_server_with_dedicated_policy_fails(self):
         """Create a pinned instance on a host with no PCPUs.
 
         This should fail because we're translating the extra spec and the host
@@ -310,10 +392,8 @@ class NUMAServersTest(NUMAServersTestBase):
         self.flags(vcpu_pin_set=None)
 
         host_info = fakelibvirt.HostInfo(cpu_nodes=1, cpu_sockets=1,
-                                         cpu_cores=5, cpu_threads=2,
-                                         kB_mem=15740000)
-        fake_connection = self._get_connection(host_info=host_info)
-        self.mock_conn.return_value = fake_connection
+                                         cpu_cores=5, cpu_threads=2)
+        self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {
             'hw:cpu_policy': 'dedicated',
@@ -322,7 +402,7 @@ class NUMAServersTest(NUMAServersTestBase):
         flavor_id = self._create_flavor(vcpu=5, extra_spec=extra_spec)
         self._run_build_test(flavor_id, end_status='ERROR')
 
-    def test_create_server_with_legacy_pinning_policy_quota_fails(self):
+    def test_create_server_with_dedicated_policy_quota_fails(self):
         """Create a pinned instance on a host with PCPUs but not enough quota.
 
         This should fail because the quota request should fail.
@@ -332,10 +412,8 @@ class NUMAServersTest(NUMAServersTestBase):
         self.flags(vcpu_pin_set=None)
 
         host_info = fakelibvirt.HostInfo(cpu_nodes=2, cpu_sockets=1,
-                                         cpu_cores=2, cpu_threads=2,
-                                         kB_mem=15740000)
-        fake_connection = self._get_connection(host_info=host_info)
-        self.mock_conn.return_value = fake_connection
+                                         cpu_cores=2, cpu_threads=2)
+        self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {
             'hw:cpu_policy': 'dedicated',
@@ -344,15 +422,7 @@ class NUMAServersTest(NUMAServersTestBase):
         flavor_id = self._create_flavor(vcpu=2, extra_spec=extra_spec)
 
         # Update the core quota less than we requested
-        self.api.update_quota({'cores': 1})
-
-        # NOTE(bhagyashris): Always use host as 'compute1' so that it's
-        # possible to get resource provider information for verifying
-        # compute usages. This host name 'compute1' is hard coded in
-        # Connection class in fakelibvirt.py.
-        # TODO(stephenfin): Remove the hardcoded limit, possibly overridding
-        # 'start_service' to make sure there isn't a mismatch
-        self.compute = self.start_service('compute', host='compute1')
+        self.admin_api.update_quota({'cores': 1})
 
         post = {'server': self._build_server(flavor_id=flavor_id)}
 
@@ -372,11 +442,8 @@ class NUMAServersTest(NUMAServersTestBase):
 
         # host has hyperthreads, which means it should be rejected
         host_info = fakelibvirt.HostInfo(
-            cpu_nodes=2, cpu_sockets=1, cpu_cores=2, cpu_threads=2,
-            kB_mem=(1024 * 1024 * 16),  # GB
-        )
-        fake_connection = self._get_connection(host_info=host_info)
-        self.mock_conn.return_value = fake_connection
+            cpu_nodes=2, cpu_sockets=1, cpu_cores=2, cpu_threads=2)
+        self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {
             'hw:cpu_policy': 'dedicated',
@@ -397,10 +464,8 @@ class NUMAServersTest(NUMAServersTestBase):
         self.flags(vcpu_pin_set=None)
 
         host_info = fakelibvirt.HostInfo(cpu_nodes=2, cpu_sockets=1,
-                                         cpu_cores=2, cpu_threads=2,
-                                         kB_mem=15740000)
-        fake_connection = self._get_connection(host_info=host_info)
-        self.mock_conn.return_value = fake_connection
+                                         cpu_cores=2, cpu_threads=2)
+        self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {'resources:PCPU': '2'}
         flavor_id = self._create_flavor(vcpu=2, extra_spec=extra_spec)
@@ -426,13 +491,12 @@ class NUMAServersTest(NUMAServersTestBase):
         self.flags(vcpu_pin_set=None)
 
         host_info = fakelibvirt.HostInfo(cpu_nodes=1, cpu_sockets=1,
-                                         cpu_cores=5, cpu_threads=2,
-                                         kB_mem=15740000)
-        fake_connection = self._get_connection(host_info=host_info)
-        self.mock_conn.return_value = fake_connection
+                                         cpu_cores=5, cpu_threads=2)
+        self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {'resources:PCPU': 2}
         flavor_id = self._create_flavor(vcpu=2, extra_spec=extra_spec)
+
         self._run_build_test(flavor_id, end_status='ERROR',
                              filter_called_on_error=False)
 
@@ -446,24 +510,14 @@ class NUMAServersTest(NUMAServersTestBase):
         self.flags(vcpu_pin_set=None)
 
         host_info = fakelibvirt.HostInfo(cpu_nodes=2, cpu_sockets=1,
-                                         cpu_cores=2, cpu_threads=2,
-                                         kB_mem=15740000)
-        fake_connection = self._get_connection(host_info=host_info)
-        self.mock_conn.return_value = fake_connection
+                                         cpu_cores=2, cpu_threads=2)
+        self.start_compute(host_info=host_info, hostname='compute1')
 
         extra_spec = {'resources:PCPU': '2'}
         flavor_id = self._create_flavor(vcpu=2, extra_spec=extra_spec)
 
         # Update the core quota less than we requested
-        self.api.update_quota({'cores': 1})
-
-        # NOTE(bhagyashris): Always use host as 'compute1' so that it's
-        # possible to get resource provider information for verifying
-        # compute usages. This host name 'compute1' is hard coded in
-        # Connection class in fakelibvirt.py.
-        # TODO(stephenfin): Remove the hardcoded limit, possibly overridding
-        # 'start_service' to make sure there isn't a mismatch
-        self.compute = self.start_service('compute', host='compute1')
+        self.admin_api.update_quota({'cores': 1})
 
         post = {'server': self._build_server(flavor_id=flavor_id)}
 
@@ -504,25 +558,11 @@ class NUMAServersTest(NUMAServersTestBase):
         self.flags(vif_plugging_timeout=0)
 
         host_info = fakelibvirt.HostInfo(cpu_nodes=2, cpu_sockets=1,
-                                         cpu_cores=2, cpu_threads=2,
-                                         kB_mem=15740000)
+                                         cpu_cores=2, cpu_threads=2)
 
         # Start services
-        self.computes = {}
-        for host in ['test_compute0', 'test_compute1']:
-            fake_connection = self._get_connection(
-                host_info=host_info, hostname=host)
-
-            # This is fun. Firstly we need to do a global'ish mock so we can
-            # actually start the service.
-            with mock.patch('nova.virt.libvirt.host.Host.get_connection',
-                            return_value=fake_connection):
-                compute = self.start_service('compute', host=host)
-
-            # Once that's done, we need to do some tweaks to each individual
-            # compute "service" to make sure they return unique objects
-            compute.driver._host.get_connection = lambda: fake_connection
-            self.computes[host] = compute
+        self.start_compute(host_info=host_info, hostname='test_compute0')
+        self.start_compute(host_info=host_info, hostname='test_compute1')
 
         # STEP ONE
 
@@ -587,7 +627,8 @@ class NUMAServersTest(NUMAServersTestBase):
         self.flags(vcpu_pin_set=None)
 
         # Start services
-        self.start_computes(save_rp_uuids=True)
+        for hostname in ('test_compute0', 'test_compute1'):
+            self.start_compute(hostname=hostname)
 
         # Create server
         flavor_a_id = self._create_flavor(extra_spec={})
@@ -603,7 +644,7 @@ class NUMAServersTest(NUMAServersTestBase):
                 expected_usage = {'VCPU': 0, 'PCPU': 0, 'DISK_GB': 0,
                                   'MEMORY_MB': 0}
 
-            compute_usage = self.placement_api.get(
+            compute_usage = self.placement.get(
                 '/resource_providers/%s/usages' % compute_rp_uuid).body[
                     'usages']
             self.assertEqual(expected_usage, compute_usage)
@@ -644,7 +685,7 @@ class NUMAServersTest(NUMAServersTestBase):
                 expected_usage = {'VCPU': 0, 'PCPU': 2, 'DISK_GB': 20,
                                   'MEMORY_MB': 2048}
 
-            compute_usage = self.placement_api.get(
+            compute_usage = self.placement.get(
                 '/resource_providers/%s/usages' % compute_rp_uuid).body[
                     'usages']
             self.assertEqual(expected_usage, compute_usage)
@@ -673,7 +714,7 @@ class NUMAServersTest(NUMAServersTestBase):
                 expected_usage = {'VCPU': 0, 'PCPU': 2, 'DISK_GB': 20,
                                   'MEMORY_MB': 2048}
 
-            compute_usage = self.placement_api.get(
+            compute_usage = self.placement.get(
                 '/resource_providers/%s/usages' % compute_rp_uuid).body[
                     'usages']
             self.assertEqual(expected_usage, compute_usage)
@@ -721,7 +762,8 @@ class NUMAServersTest(NUMAServersTestBase):
         )
 
         # start services
-        self.start_computes(save_rp_uuids=True)
+        self.start_compute(hostname='test_compute0')
+        self.start_compute(hostname='test_compute1')
 
         # create server
         flavor_a_id = self._create_flavor(
@@ -797,17 +839,14 @@ class ReshapeForPCPUsTest(NUMAServersTestBase):
                    group='compute')
         self.flags(vcpu_pin_set='0-7')
 
-        host_info = fakelibvirt.HostInfo(cpu_nodes=2, cpu_sockets=1,
-                                         cpu_cores=2, cpu_threads=2,
-                                         kB_mem=15740000)
-
-        # Start services
-        self.start_computes(save_rp_uuids=True)
+        # start services
+        self.start_compute(hostname='test_compute0')
+        self.start_compute(hostname='test_compute1')
 
         # ensure there is no PCPU inventory being reported
 
         for host, compute_rp_uuid in self.compute_rp_uuids.items():
-            compute_inventory = self.placement_api.get(
+            compute_inventory = self.placement.get(
                 '/resource_providers/%s/inventories' % compute_rp_uuid).body[
                     'inventories']
             self.assertEqual(8, compute_inventory['VCPU']['total'])
@@ -833,20 +872,20 @@ class ReshapeForPCPUsTest(NUMAServersTestBase):
 
         compute_rp_uuid = self.compute_rp_uuids['test_compute0']
 
-        compute_inventory = self.placement_api.get(
+        compute_inventory = self.placement.get(
             '/resource_providers/%s/inventories' % compute_rp_uuid).body[
                 'inventories']
-        compute_usages = self.placement_api.get(
+        compute_usages = self.placement.get(
             '/resource_providers/%s/usages' % compute_rp_uuid).body[
                 'usages']
         self.assertEqual(4, compute_usages['VCPU'])
 
         compute_rp_uuid = self.compute_rp_uuids['test_compute1']
 
-        compute_inventory = self.placement_api.get(
+        compute_inventory = self.placement.get(
             '/resource_providers/%s/inventories' % compute_rp_uuid).body[
                 'inventories']
-        compute_usages = self.placement_api.get(
+        compute_usages = self.placement.get(
             '/resource_providers/%s/usages' % compute_rp_uuid).body[
                 'usages']
         self.assertEqual(0, compute_usages['VCPU'])
@@ -870,18 +909,18 @@ class ReshapeForPCPUsTest(NUMAServersTestBase):
 
         compute_rp_uuid = self.compute_rp_uuids['test_compute0']
 
-        compute_inventory = self.placement_api.get(
+        compute_inventory = self.placement.get(
             '/resource_providers/%s/inventories' % compute_rp_uuid).body[
                 'inventories']
         self.assertEqual(8, compute_inventory['VCPU']['total'])
         self.assertNotIn('PCPU', compute_inventory)
-        compute_usages = self.placement_api.get(
+        compute_usages = self.placement.get(
             '/resource_providers/%s/usages' % compute_rp_uuid).body[
                 'usages']
         self.assertEqual(4, compute_usages['VCPU'])
         self.assertNotIn('PCPU', compute_usages)
 
-        allocations = self.placement_api.get(
+        allocations = self.placement.get(
             '/allocations/%s' % server1['id']).body['allocations']
         # the flavor has disk=10 and ephemeral=10
         self.assertEqual(
@@ -893,18 +932,18 @@ class ReshapeForPCPUsTest(NUMAServersTestBase):
 
         compute_rp_uuid = self.compute_rp_uuids['test_compute1']
 
-        compute_inventory = self.placement_api.get(
+        compute_inventory = self.placement.get(
             '/resource_providers/%s/inventories' % compute_rp_uuid).body[
                 'inventories']
         self.assertEqual(8, compute_inventory['VCPU']['total'])
         self.assertNotIn('PCPU', compute_inventory)
-        compute_usages = self.placement_api.get(
+        compute_usages = self.placement.get(
             '/resource_providers/%s/usages' % compute_rp_uuid).body[
                 'usages']
         self.assertEqual(2, compute_usages['VCPU'])
         self.assertNotIn('PCPU', compute_usages)
 
-        allocations = self.placement_api.get(
+        allocations = self.placement.get(
             '/allocations/%s' % server2['id']).body['allocations']
         # the flavor has disk=10 and ephemeral=10
         self.assertEqual(
@@ -918,22 +957,10 @@ class ReshapeForPCPUsTest(NUMAServersTestBase):
         self.flags(cpu_dedicated_set='0-7', group='compute')
         self.flags(vcpu_pin_set=None)
 
-        for host in ['test_compute0', 'test_compute1']:
-            self.computes[host].stop()
-
-            fake_connection = self._get_connection(
-                host_info=host_info, hostname=host)
-
-            # This is fun. Firstly we need to do a global'ish mock so we can
-            # actually start the service.
-            with mock.patch('nova.virt.libvirt.host.Host.get_connection',
-                            return_value=fake_connection):
-                compute = self.start_service('compute', host=host)
-
-            # Once that's done, we need to do some tweaks to each individual
-            # compute "service" to make sure they return unique objects
-            compute.driver._host.get_connection = lambda: fake_connection
-            self.computes[host] = compute
+        computes = {}
+        for host, compute in self.computes.items():
+            computes[host] = self.restart_compute_service(compute)
+        self.computes = computes
 
         # verify that the inventory, usages and allocation are correct after
         # the reshape
@@ -944,18 +971,18 @@ class ReshapeForPCPUsTest(NUMAServersTestBase):
 
         compute_rp_uuid = self.compute_rp_uuids['test_compute0']
 
-        compute_inventory = self.placement_api.get(
+        compute_inventory = self.placement.get(
             '/resource_providers/%s/inventories' % compute_rp_uuid).body[
                 'inventories']
         self.assertEqual(8, compute_inventory['PCPU']['total'])
         self.assertNotIn('VCPU', compute_inventory)
-        compute_usages = self.placement_api.get(
+        compute_usages = self.placement.get(
             '/resource_providers/%s/usages' % compute_rp_uuid).body[
                 'usages']
         self.assertEqual(4, compute_usages['PCPU'])
         self.assertNotIn('VCPU', compute_usages)
 
-        allocations = self.placement_api.get(
+        allocations = self.placement.get(
             '/allocations/%s' % server1['id']).body['allocations']
         # the flavor has disk=10 and ephemeral=10
         self.assertEqual(
@@ -967,18 +994,18 @@ class ReshapeForPCPUsTest(NUMAServersTestBase):
 
         compute_rp_uuid = self.compute_rp_uuids['test_compute1']
 
-        compute_inventory = self.placement_api.get(
+        compute_inventory = self.placement.get(
             '/resource_providers/%s/inventories' % compute_rp_uuid).body[
                 'inventories']
         self.assertEqual(8, compute_inventory['PCPU']['total'])
         self.assertNotIn('VCPU', compute_inventory)
-        compute_usages = self.placement_api.get(
+        compute_usages = self.placement.get(
             '/resource_providers/%s/usages' % compute_rp_uuid).body[
                 'usages']
         self.assertEqual(2, compute_usages['PCPU'])
         self.assertNotIn('VCPU', compute_usages)
 
-        allocations = self.placement_api.get(
+        allocations = self.placement.get(
             '/allocations/%s' % server2['id']).body['allocations']
         # the flavor has disk=10 and ephemeral=10
         self.assertEqual(
@@ -993,12 +1020,12 @@ class ReshapeForPCPUsTest(NUMAServersTestBase):
 
         compute_rp_uuid = self.compute_rp_uuids['test_compute0']
 
-        compute_inventory = self.placement_api.get(
+        compute_inventory = self.placement.get(
             '/resource_providers/%s/inventories' % compute_rp_uuid).body[
                 'inventories']
         self.assertEqual(8, compute_inventory['PCPU']['total'])
         self.assertNotIn('VCPU', compute_inventory)
-        compute_usages = self.placement_api.get(
+        compute_usages = self.placement.get(
             '/resource_providers/%s/usages' % compute_rp_uuid).body[
                 'usages']
         self.assertEqual(6, compute_usages['PCPU'])
@@ -1006,7 +1033,7 @@ class ReshapeForPCPUsTest(NUMAServersTestBase):
 
         # check the allocations for this server specifically
 
-        allocations = self.placement_api.get(
+        allocations = self.placement.get(
             '/allocations/%s' % server3['id']).body[
                 'allocations']
         self.assertEqual(
@@ -1038,13 +1065,7 @@ class NUMAServersWithNetworksTest(NUMAServersTestBase):
 
     def _test_create_server_with_networks(self, flavor_id, networks,
                                           end_status='ACTIVE'):
-        host_info = fakelibvirt.HostInfo(cpu_nodes=2, cpu_sockets=1,
-                                         cpu_cores=2, cpu_threads=2,
-                                         kB_mem=15740000)
-        fake_connection = self._get_connection(host_info=host_info)
-        self.mock_conn.return_value = fake_connection
-
-        self.compute = self.start_service('compute', host='test_compute0')
+        self.start_compute()
 
         # Create server
         return self._create_server(
@@ -1175,12 +1196,13 @@ class NUMAServersWithNetworksTest(NUMAServersTestBase):
         # is a cast, but since we are using CastAsCall this will bubble to the
         # API.
         self.assertEqual(500, ex.response.status_code)
-        self.assertIn('NoValidHost', six.text_type(ex))
+        self.assertIn('NoValidHost', str(ex))
 
     def test_cold_migrate_with_physnet(self):
 
         # Start services
-        self.start_computes(save_rp_uuids=True)
+        self.start_compute(hostname='test_compute0')
+        self.start_compute(hostname='test_compute1')
 
         # Create server
         extra_spec = {'hw:numa_nodes': '1'}
@@ -1204,7 +1226,7 @@ class NUMAServersWithNetworksTest(NUMAServersTestBase):
         # probably be less...dumb
         with mock.patch('nova.virt.libvirt.driver.LibvirtDriver'
                         '.migrate_disk_and_power_off', return_value='{}'):
-            self.api.post_server_action(server['id'], {'migrate': None})
+            self.admin_api.post_server_action(server['id'], {'migrate': None})
 
         server = self._wait_for_state_change(server, 'VERIFY_RESIZE')
 
@@ -1247,13 +1269,6 @@ class NUMAServersRebuildTests(NUMAServersTestBase):
         server = self.api.post_server({'server': basic_server})
         return self._wait_for_state_change(server, 'ACTIVE')
 
-    def _rebuild_server(self, active_server, image_ref):
-        args = {"rebuild": {"imageRef": image_ref}}
-        self.api.api_post(
-            'servers/%s/action' % active_server['id'], args)
-        fake_notifier.wait_for_versioned_notifications('instance.rebuild.end')
-        return self._wait_for_state_change(active_server, 'ACTIVE')
-
     def test_rebuild_server_with_numa(self):
         """Create a NUMA instance and ensure it can be rebuilt.
         """
@@ -1267,10 +1282,8 @@ class NUMAServersRebuildTests(NUMAServersTestBase):
         # the free space to ensure the numa topology filter does not
         # eliminate the host.
         host_info = fakelibvirt.HostInfo(cpu_nodes=1, cpu_sockets=1,
-                                         cpu_cores=4, kB_mem=15740000)
-        fake_connection = self._get_connection(host_info=host_info)
-        self.mock_conn.return_value = fake_connection
-        self.compute = self.start_service('compute', host='compute1')
+                                         cpu_cores=4)
+        self.start_compute(host_info=host_info)
 
         server = self._create_active_server(
             server_args={"flavorRef": flavor_id})
@@ -1293,11 +1306,8 @@ class NUMAServersRebuildTests(NUMAServersTestBase):
         # cpu_cores is set to 2 to ensure that we have enough space
         # to boot the vm but not enough space to rebuild
         # by doubling the resource use during scheduling.
-        host_info = fakelibvirt.HostInfo(
-            cpu_nodes=1, cpu_sockets=1, cpu_cores=2, kB_mem=15740000)
-        fake_connection = self._get_connection(host_info=host_info)
-        self.mock_conn.return_value = fake_connection
-        self.compute = self.start_service('compute', host='compute1')
+        host_info = fakelibvirt.HostInfo()
+        self.start_compute(host_info=host_info)
 
         server = self._create_active_server(
             server_args={"flavorRef": flavor_id})
@@ -1315,10 +1325,8 @@ class NUMAServersRebuildTests(NUMAServersTestBase):
         flavor_id = self._create_flavor(extra_spec=extra_spec)
 
         host_info = fakelibvirt.HostInfo(
-            cpu_nodes=2, cpu_sockets=1, cpu_cores=4, kB_mem=15740000)
-        fake_connection = self._get_connection(host_info=host_info)
-        self.mock_conn.return_value = fake_connection
-        self.compute = self.start_service('compute', host='compute1')
+            cpu_nodes=2, cpu_sockets=1, cpu_cores=4)
+        self.start_compute(host_info=host_info)
 
         server = self._create_active_server(
             server_args={"flavorRef": flavor_id})
@@ -1328,7 +1336,7 @@ class NUMAServersRebuildTests(NUMAServersTestBase):
         # 2 virtual numa nodes.
         ctx = nova_context.get_admin_context()
         image_meta = {'properties': {'hw_numa_nodes': 2}}
-        self.fake_image_service.update(ctx, self.image_ref_1, image_meta)
+        self.glance.update(ctx, self.image_ref_1, image_meta)
 
         # NOTE(sean-k-mooney): this should fail because rebuild uses noop
         # claims therefore it is not allowed for the NUMA topology or resource
@@ -1337,5 +1345,4 @@ class NUMAServersRebuildTests(NUMAServersTestBase):
             client.OpenStackApiException, self._rebuild_server,
             server, self.image_ref_1)
         self.assertEqual(400, ex.response.status_code)
-        self.assertIn("An instance's NUMA topology cannot be changed",
-                      six.text_type(ex))
+        self.assertIn("An instance's NUMA topology cannot be changed", str(ex))
