@@ -18,20 +18,21 @@
 A driver wrapping the Ironic API, such that Nova may provision
 bare metal resources.
 """
+
 import base64
 from distutils import version
 import gzip
 import shutil
 import tempfile
 import time
+from urllib import parse as urlparse
 
+from openstack import exceptions as sdk_exc
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_service import loopingcall
 from oslo_utils import excutils
 from oslo_utils import importutils
-import six
-import six.moves.urllib.parse as urlparse
 from tooz import hashring as hash_ring
 
 from nova.api.metadata import base as instance_metadata
@@ -45,12 +46,12 @@ from nova import context as nova_context
 from nova import exception
 from nova.i18n import _
 from nova import objects
+from nova.objects import external_event as external_event_obj
 from nova.objects import fields as obj_fields
 from nova import servicegroup
 from nova import utils
 from nova.virt import configdrive
 from nova.virt import driver as virt_driver
-from nova.virt import firewall
 from nova.virt import hardware
 from nova.virt.ironic import client_wrapper
 from nova.virt.ironic import ironic_states
@@ -154,13 +155,29 @@ def _check_peer_list():
 class IronicDriver(virt_driver.ComputeDriver):
     """Hypervisor driver for Ironic - bare metal provisioning."""
 
-    capabilities = {"has_imagecache": False,
-                    "supports_evacuate": False,
-                    "supports_migrate_to_same_host": False,
-                    "supports_attach_interface": True,
-                    "supports_multiattach": False,
-                    "supports_trusted_certs": False,
-                    }
+    capabilities = {
+        "has_imagecache": False,
+        "supports_evacuate": False,
+        "supports_migrate_to_same_host": False,
+        "supports_attach_interface": True,
+        "supports_multiattach": False,
+        "supports_trusted_certs": False,
+        "supports_pcpus": False,
+        "supports_accelerators": False,
+
+        # Image type support flags
+        "supports_image_type_aki": False,
+        "supports_image_type_ami": False,
+        "supports_image_type_ari": False,
+        "supports_image_type_iso": False,
+        "supports_image_type_qcow2": True,
+        "supports_image_type_raw": True,
+        "supports_image_type_vdi": False,
+        "supports_image_type_vhd": False,
+        "supports_image_type_vhdx": False,
+        "supports_image_type_vmdk": False,
+        "supports_image_type_ploop": False,
+    }
 
     # This driver is capable of rebalancing nodes between computes.
     rebalances_nodes = True
@@ -177,27 +194,35 @@ class IronicDriver(virt_driver.ComputeDriver):
                 ironic.client = importutils.import_module(
                                                     'ironicclient.client')
 
-        self.firewall_driver = firewall.load_driver(
-            default='nova.virt.firewall.NoopFirewallDriver')
         self.node_cache = {}
         self.node_cache_time = 0
         self.servicegroup_api = servicegroup.API()
 
         self.ironicclient = client_wrapper.IronicClientWrapper()
+        self._ironic_connection = None
 
-        # This is needed for the instance flavor migration in Pike, and should
-        # be removed in Queens. Since this will run several times in the life
-        # of the driver, track the instances that have already been migrated.
-        self._migrated_instance_uuids = set()
+    @property
+    def ironic_connection(self):
+        if self._ironic_connection is None:
+            # Ask get_sdk_adapter to raise ServiceUnavailable if the baremetal
+            # service isn't ready yet. Consumers of ironic_connection are set
+            # up to handle this and raise VirtDriverNotReady as appropriate.
+            self._ironic_connection = utils.get_sdk_adapter(
+                'baremetal', check_service=True)
+        return self._ironic_connection
 
-    def _get_node(self, node_uuid):
+    def _get_node(self, node_id):
         """Get a node by its UUID.
 
            Some methods pass in variables named nodename, but are
            actually UUID's.
         """
-        return self.ironicclient.call('node.get', node_uuid,
-                                      fields=_NODE_FIELDS)
+        node = self.ironic_connection.get_node(node_id, fields=_NODE_FIELDS)
+        # TODO(dustinc): Make consumers use the right fields and remove this
+        node.uuid = node.id
+        node.instance_uuid = node.instance_id
+        node.maintenance = node.is_maintenance
+        return node
 
     def _validate_instance_and_node(self, instance):
         """Get the node associated with the instance.
@@ -205,11 +230,25 @@ class IronicDriver(virt_driver.ComputeDriver):
         Check with the Ironic service that this instance is associated with a
         node, and return the node.
         """
-        try:
-            return self.ironicclient.call('node.get_by_instance_uuid',
-                                          instance.uuid, fields=_NODE_FIELDS)
-        except ironic.exc.NotFound:
+        nodes = list(self.ironic_connection.nodes(
+            instance_id=instance.uuid, fields=_NODE_FIELDS))
+        if not nodes:
             raise exception.InstanceNotFound(instance_id=instance.uuid)
+        if len(nodes) > 1:
+            # This indicates a programming error so fail.
+            raise exception.NovaException(
+                _('Ironic returned more than one node for a query '
+                  'that can only return zero or one: %s') % nodes)
+
+        node = nodes[0]
+        # TODO(dustinc): Make consumers use the right fields and remove
+        if hasattr(node, "id"):
+            node.uuid = node.id
+        if hasattr(node, "instance_id"):
+            node.instance_uuid = node.instance_id
+        if hasattr(node, "is_maintenance"):
+            node.maintenance = node.is_maintenance
+        return node
 
     def _node_resources_unavailable(self, node_obj):
         """Determine whether the node's resources are in an acceptable state.
@@ -335,28 +374,17 @@ class IronicDriver(virt_driver.ComputeDriver):
         }
         return dic
 
-    def _start_firewall(self, instance, network_info):
-        self.firewall_driver.setup_basic_filtering(instance, network_info)
-        self.firewall_driver.prepare_instance_filter(instance, network_info)
-        self.firewall_driver.apply_instance_filter(instance, network_info)
-
-    def _stop_firewall(self, instance, network_info):
-        self.firewall_driver.unfilter_instance(instance, network_info)
-
-    def _set_instance_uuid(self, node, instance):
-
-        patch = [{'path': '/instance_uuid', 'op': 'add',
-                  'value': instance.uuid}]
+    def _set_instance_id(self, node, instance):
         try:
-            # NOTE(TheJulia): Assert an instance UUID to lock the node
+            # NOTE(TheJulia): Assert an instance ID to lock the node
             # from other deployment attempts while configuration is
             # being set.
-            self.ironicclient.call('node.update', node.uuid, patch,
-                                   retry_on_conflict=False)
-        except ironic.exc.BadRequest:
+            self.ironic_connection.update_node(node, retry_on_conflict=False,
+                                               instance_id=instance.uuid)
+        except sdk_exc.SDKException:
             msg = (_("Failed to reserve node %(node)s "
                      "when provisioning the instance %(instance)s")
-                   % {'node': node.uuid, 'instance': instance.uuid})
+                   % {'node': node.id, 'instance': instance.uuid})
             LOG.error(msg)
             raise exception.InstanceDeployFailure(msg)
 
@@ -368,7 +396,7 @@ class IronicDriver(virt_driver.ComputeDriver):
                 _("Ironic node uuid not supplied to "
                   "driver for instance %s.") % instance.uuid)
         node = self._get_node(node_uuid)
-        self._set_instance_uuid(node, instance)
+        self._set_instance_id(node, instance)
 
     def failed_spawn_cleanup(self, instance):
         LOG.debug('Failed spawn cleanup called for instance',
@@ -419,7 +447,7 @@ class IronicDriver(virt_driver.ComputeDriver):
                         "%(node)s when unprovisioning the instance "
                         "%(instance)s: %(reason)s",
                         {'node': node.uuid, 'instance': instance.uuid,
-                         'reason': six.text_type(e)})
+                         'reason': str(e)})
 
     def _add_volume_target_info(self, context, instance, block_device_info):
         bdms = virt_driver.block_device_info_get_mapping(block_device_info)
@@ -478,7 +506,6 @@ class IronicDriver(virt_driver.ComputeDriver):
                         remove_instance_info=True):
         self._cleanup_volume_target_info(instance)
         self._unplug_vifs(node, instance, network_info)
-        self._stop_firewall(instance, network_info)
         if remove_instance_info:
             self._remove_instance_info_from_node(node, instance)
 
@@ -534,55 +561,6 @@ class IronicDriver(virt_driver.ComputeDriver):
         """
         self._refresh_hash_ring(nova_context.get_admin_context())
 
-    @staticmethod
-    def _pike_flavor_migration_for_node(ctx, node_rc, instance_uuid):
-        normalized_rc = utils.normalize_rc_name(node_rc)
-        instance = objects.Instance.get_by_uuid(ctx, instance_uuid,
-                                                expected_attrs=["flavor"])
-        specs = instance.flavor.extra_specs
-        resource_key = "resources:%s" % normalized_rc
-        if resource_key in specs:
-            # The compute must have been restarted, and the instance.flavor
-            # has already been migrated
-            return False
-        specs[resource_key] = "1"
-        instance.save()
-        return True
-
-    def _pike_flavor_migration(self, node_uuids):
-        """This code is needed in Pike to prevent problems where an operator
-        has already adjusted their flavors to add the custom resource class to
-        extra_specs. Since existing ironic instances will not have this in
-        their extra_specs, they will only have allocations against
-        VCPU/RAM/disk. By adding just the custom RC to the existing flavor
-        extra_specs, the periodic call to update_available_resources() will add
-        an allocation against the custom resource class, and prevent placement
-        from thinking that that node is available. This code can be removed in
-        Queens, and will need to be updated to also alter extra_specs to
-        zero-out the old-style standard resource classes of VCPU, MEMORY_MB,
-        and DISK_GB.
-        """
-        ctx = nova_context.get_admin_context()
-
-        for node_uuid in node_uuids:
-            node = self._node_from_cache(node_uuid)
-            if not node:
-                continue
-            node_rc = node.resource_class
-            if not node_rc:
-                LOG.warning("Node %(node)s does not have its resource_class "
-                        "set.", {"node": node.uuid})
-                continue
-            if node.instance_uuid in self._migrated_instance_uuids:
-                continue
-            self._pike_flavor_migration_for_node(ctx, node_rc,
-                                                 node.instance_uuid)
-            self._migrated_instance_uuids.add(node.instance_uuid)
-            LOG.debug("The flavor extra_specs for Ironic instance %(inst)s "
-                      "have been updated for custom resource class '%(rc)s'.",
-                      {"inst": node.instance_uuid, "rc": node_rc})
-        return
-
     def _get_hypervisor_type(self):
         """Get hypervisor type."""
         return 'ironic'
@@ -607,27 +585,47 @@ class IronicDriver(virt_driver.ComputeDriver):
         except exception.InstanceNotFound:
             return False
 
-    def _get_node_list(self, **kwargs):
-        """Helper function to return the list of nodes.
+    def _get_node_list(self, return_generator=False, **kwargs):
+        """Helper function to return a list or generator of nodes.
 
-        If unable to connect ironic server, an empty list is returned.
-
-        :returns: a list of raw node from ironic
+        :param return_generator: If True, returns a generator of nodes. This
+          generator will only have SDK attribute names.
+        :returns: a list or generator of raw nodes from ironic
         :raises: VirtDriverNotReady
-
         """
-        node_list = []
         try:
-            node_list = self.ironicclient.call("node.list", **kwargs)
-        except exception.NovaException as e:
-            LOG.error("Failed to get the list of nodes from the Ironic "
-                      "inventory. Error: %s", e)
+            # NOTE(dustinc): The generator returned by the SDK can only be
+            # interated once. Since there are cases where it needs to be
+            # iterated more than once, we should return it as a list. In the
+            # future it may be worth refactoring these other usages so it can
+            # be returned as a generator.
+            node_generator = self.ironic_connection.nodes(**kwargs)
+        except sdk_exc.InvalidResourceQuery as e:
+            LOG.error("Invalid parameters in the provided search query."
+                      "Error: %s", str(e))
             raise exception.VirtDriverNotReady()
         except Exception as e:
             LOG.error("An unknown error has occurred when trying to get the "
-                      "list of nodes from the Ironic inventory. Error: %s", e)
+                      "list of nodes from the Ironic inventory. Error: %s",
+                      str(e))
             raise exception.VirtDriverNotReady()
-        return node_list
+        if return_generator:
+            return node_generator
+        else:
+            node_list = []
+            # TODO(dustinc): Update all usages to use SDK attributes then stop
+            #  copying values to PythonClient attributes.
+            for node in node_generator:
+                # NOTE(dustinc): There are usages that filter out these fields
+                #  which forces us to check for the attributes.
+                if hasattr(node, "id"):
+                    node.uuid = node.id
+                if hasattr(node, "instance_id"):
+                    node.instance_uuid = node.instance_id
+                if hasattr(node, "is_maintenance"):
+                    node.maintenance = node.is_maintenance
+                node_list.append(node)
+            return node_list
 
     def list_instances(self):
         """Return the names of all the instances provisioned.
@@ -636,27 +634,25 @@ class IronicDriver(virt_driver.ComputeDriver):
         :raises: VirtDriverNotReady
 
         """
-        # NOTE(lucasagomes): limit == 0 is an indicator to continue
-        # pagination until there're no more values to be returned.
-        node_list = self._get_node_list(associated=True,
-                                        fields=['instance_uuid'], limit=0)
+        # NOTE(dustinc): The SDK returns an object with instance_id,
+        #  but the Ironic API expects instance_uuid in query.
         context = nova_context.get_admin_context()
-        return [objects.Instance.get_by_uuid(context,
-                                             i.instance_uuid).name
-                for i in node_list]
+        return [objects.Instance.get_by_uuid(context, i.instance_id).name
+                for i in self._get_node_list(return_generator=True,
+                                             associated=True,
+                                             fields=['instance_uuid'])]
 
     def list_instance_uuids(self):
-        """Return the UUIDs of all the instances provisioned.
+        """Return the IDs of all the instances provisioned.
 
-        :returns: a list of instance UUIDs.
+        :returns: a list of instance IDs.
         :raises: VirtDriverNotReady
 
         """
-        # NOTE(lucasagomes): limit == 0 is an indicator to continue
-        # pagination until there're no more values to be returned.
-        node_list = self._get_node_list(associated=True,
-                                        fields=['instance_uuid'], limit=0)
-        return list(n.instance_uuid for n in node_list)
+        # NOTE(dustinc): The SDK returns an object with instance_id,
+        #  but the Ironic API expects instance_uuid in query.
+        return [node.instance_id for node in self._get_node_list(
+            return_generator=True, associated=True, fields=['instance_uuid'])]
 
     def node_is_available(self, nodename):
         """Confirms a Nova hypervisor node exists in the Ironic inventory.
@@ -687,7 +683,7 @@ class IronicDriver(virt_driver.ComputeDriver):
             # nodename is the ironic node's UUID.
             self._get_node(nodename)
             return True
-        except ironic.exc.NotFound:
+        except sdk_exc.ResourceNotFound:
             return False
 
     def _refresh_hash_ring(self, ctxt):
@@ -735,15 +731,13 @@ class IronicDriver(virt_driver.ComputeDriver):
         LOG.debug('Hash ring members are %s', services)
 
     def _refresh_cache(self):
-        # NOTE(lucasagomes): limit == 0 is an indicator to continue
-        # pagination until there're no more values to be returned.
         ctxt = nova_context.get_admin_context()
         self._refresh_hash_ring(ctxt)
         instances = objects.InstanceList.get_uuids_by_host(ctxt, CONF.host)
         node_cache = {}
 
         def _get_node_list(**kwargs):
-            return self._get_node_list(fields=_NODE_FIELDS, limit=0, **kwargs)
+            return self._get_node_list(fields=_NODE_FIELDS, **kwargs)
 
         # NOTE(jroll) if partition_key is set, we need to limit nodes that
         # can be managed to nodes that have a matching conductor_group
@@ -784,16 +778,6 @@ class IronicDriver(virt_driver.ComputeDriver):
 
         self.node_cache = node_cache
         self.node_cache_time = time.time()
-        # For Pike, we need to ensure that all instances have their flavor
-        # migrated to include the resource_class. Since there could be many,
-        # many instances controlled by this host, spawn this asynchronously so
-        # as not to block this service.
-        node_uuids = [node.uuid for node in self.node_cache.values()
-                      if node.instance_uuid and
-                      node.instance_uuid not in self._migrated_instance_uuids]
-        if node_uuids:
-            # No need to run unless something has changed
-            utils.spawn_n(self._pike_flavor_migration, node_uuids)
 
     def get_available_nodes(self, refresh=False):
         """Returns the UUIDs of Ironic nodes managed by this compute service.
@@ -991,14 +975,6 @@ class IronicDriver(virt_driver.ComputeDriver):
 
         return hardware.InstanceInfo(state=map_power_state(node.power_state))
 
-    def deallocate_networks_on_reschedule(self, instance):
-        """Does the driver want networks deallocated on reschedule?
-
-        :param instance: the instance object.
-        :returns: Boolean value. If True deallocate networks on reschedule.
-        """
-        return True
-
     def _get_network_metadata(self, node, network_info):
         """Gets a more complete representation of the instance network info.
 
@@ -1083,8 +1059,7 @@ class IronicDriver(virt_driver.ComputeDriver):
 
         i_meta = instance_metadata.InstanceMetadata(instance,
             content=files, extra_md=extra_md, network_info=network_info,
-            network_metadata=self._get_network_metadata(node, network_info),
-            request_context=context)
+            network_metadata=self._get_network_metadata(node, network_info))
 
         with tempfile.NamedTemporaryFile() as uncompressed:
             with configdrive.ConfigDriveBuilder(instance_md=i_meta) as cdb:
@@ -1102,7 +1077,7 @@ class IronicDriver(virt_driver.ComputeDriver):
 
     def spawn(self, context, instance, image_meta, injected_files,
               admin_password, allocations, network_info=None,
-              block_device_info=None):
+              block_device_info=None, power_on=True, accel_info=None):
         """Deploy an instance.
 
         :param context: The security context.
@@ -1119,6 +1094,9 @@ class IronicDriver(virt_driver.ComputeDriver):
         :param network_info: Instance network information.
         :param block_device_info: Instance block device
             information.
+        :param arqs: Accelerator requests for this instance.
+        :param power_on: True if the instance should be powered on, False
+                         otherwise
         """
         LOG.debug('Spawn called for instance', instance=instance)
 
@@ -1155,9 +1133,9 @@ class IronicDriver(virt_driver.ComputeDriver):
 
         # validate we are ready to do the deploy
         validate_chk = self.ironicclient.call("node.validate", node_uuid)
-        if (not validate_chk.deploy.get('result')
-                or not validate_chk.power.get('result')
-                or not validate_chk.storage.get('result')):
+        if (not validate_chk.deploy.get('result') or
+                not validate_chk.power.get('result') or
+                not validate_chk.storage.get('result')):
             # something is wrong. undo what we have done
             self._cleanup_deploy(node, instance, network_info)
             raise exception.ValidationError(_(
@@ -1169,17 +1147,6 @@ class IronicDriver(virt_driver.ComputeDriver):
                    'power': validate_chk.power,
                    'storage': validate_chk.storage})
 
-        # prepare for the deploy
-        try:
-            self._start_firewall(instance, network_info)
-        except Exception:
-            with excutils.save_and_reraise_exception():
-                LOG.error("Error preparing deploy for instance "
-                          "%(instance)s on baremetal node %(node)s.",
-                          {'instance': instance.uuid,
-                           'node': node_uuid})
-                self._cleanup_deploy(node, instance, network_info)
-
         # Config drive
         configdrive_value = None
         if configdrive.required_by(instance):
@@ -1193,8 +1160,7 @@ class IronicDriver(virt_driver.ComputeDriver):
                     files=injected_files)
             except Exception as e:
                 with excutils.save_and_reraise_exception():
-                    msg = ("Failed to build configdrive: %s" %
-                           six.text_type(e))
+                    msg = "Failed to build configdrive: %s" % str(e)
                     LOG.error(msg, instance=instance)
                     self._cleanup_deploy(node, instance, network_info)
 
@@ -1212,7 +1178,7 @@ class IronicDriver(virt_driver.ComputeDriver):
                 LOG.error("Failed to request Ironic to provision instance "
                           "%(inst)s: %(reason)s",
                           {'inst': instance.uuid,
-                           'reason': six.text_type(e)})
+                           'reason': str(e)})
                 self._cleanup_deploy(node, instance, network_info)
 
         timer = loopingcall.FixedIntervalLoopingCall(self._wait_for_active,
@@ -1340,7 +1306,8 @@ class IronicDriver(virt_driver.ComputeDriver):
                  node.uuid, instance=instance)
 
     def reboot(self, context, instance, network_info, reboot_type,
-               block_device_info=None, bad_volumes_callback=None):
+               block_device_info=None, bad_volumes_callback=None,
+               accel_info=None):
         """Reboot the specified instance.
 
         NOTE: Unlike the libvirt driver, this method does not delete
@@ -1355,7 +1322,8 @@ class IronicDriver(virt_driver.ComputeDriver):
             Ignored by this driver.
         :param bad_volumes_callback: Function to handle any bad volumes
             encountered. Ignored by this driver.
-
+        :param accel_info: List of accelerator request dicts. The exact
+            data struct is doc'd in nova/virt/driver.py::spawn().
         """
         LOG.debug('Reboot(type %s) called for instance',
                   reboot_type, instance=instance)
@@ -1441,7 +1409,7 @@ class IronicDriver(virt_driver.ComputeDriver):
                  node.uuid, instance=instance)
 
     def power_on(self, context, instance, network_info,
-                 block_device_info=None):
+                 block_device_info=None, accel_info=None):
         """Power on the specified instance.
 
         NOTE: Unlike the libvirt driver, this method does not delete
@@ -1453,7 +1421,8 @@ class IronicDriver(virt_driver.ComputeDriver):
             this driver.
         :param block_device_info: Instance block device
             information. Ignored by this driver.
-
+        :param accel_info: List of accelerator requests for this instance.
+                           Ignored by this driver.
         """
         LOG.debug('Power on called for instance', instance=instance)
         node = self._validate_instance_and_node(instance)
@@ -1465,6 +1434,26 @@ class IronicDriver(virt_driver.ComputeDriver):
         LOG.info('Successfully powered on Ironic node %s',
                  node.uuid, instance=instance)
 
+    def power_update_event(self, instance, target_power_state):
+        """Update power, vm and task states of the specified instance in
+        the nova DB.
+        """
+        LOG.info('Power update called for instance with '
+                 'target power state %s.', target_power_state,
+                 instance=instance)
+        if target_power_state == external_event_obj.POWER_ON:
+            instance.power_state = power_state.RUNNING
+            instance.vm_state = vm_states.ACTIVE
+            instance.task_state = None
+            expected_task_state = task_states.POWERING_ON
+        else:
+            # It's POWER_OFF
+            instance.power_state = power_state.SHUTDOWN
+            instance.vm_state = vm_states.STOPPED
+            instance.task_state = None
+            expected_task_state = task_states.POWERING_OFF
+        instance.save(expected_task_state=expected_task_state)
+
     def trigger_crash_dump(self, instance):
         """Trigger crash dump mechanism on the given instance.
 
@@ -1483,47 +1472,6 @@ class IronicDriver(virt_driver.ComputeDriver):
         LOG.info('Successfully triggered crash dump into Ironic node %s',
                  node.uuid, instance=instance)
 
-    def refresh_security_group_rules(self, security_group_id):
-        """Refresh security group rules from data store.
-
-        Invoked when security group rules are updated.
-
-        :param security_group_id: The security group id.
-
-        """
-        self.firewall_driver.refresh_security_group_rules(security_group_id)
-
-    def refresh_instance_security_rules(self, instance):
-        """Refresh security group rules from data store.
-
-        Gets called when an instance gets added to or removed from
-        the security group the instance is a member of or if the
-        group gains or loses a rule.
-
-        :param instance: The instance object.
-
-        """
-        self.firewall_driver.refresh_instance_security_rules(instance)
-
-    def ensure_filtering_rules_for_instance(self, instance, network_info):
-        """Set up filtering rules.
-
-        :param instance: The instance object.
-        :param network_info: Instance network information.
-
-        """
-        self.firewall_driver.setup_basic_filtering(instance, network_info)
-        self.firewall_driver.prepare_instance_filter(instance, network_info)
-
-    def unfilter_instance(self, instance, network_info):
-        """Stop filtering instance.
-
-        :param instance: The instance object.
-        :param network_info: Instance network information.
-
-        """
-        self.firewall_driver.unfilter_instance(instance, network_info)
-
     def _plug_vif(self, node, port_id):
         last_attempt = 5
         for attempt in range(0, last_attempt + 1):
@@ -1540,8 +1488,7 @@ class IronicDriver(virt_driver.ComputeDriver):
                 # which will cause ironicclient to automatically retry for us.
                 # We can remove this workaround once we are confident that we
                 # are only running against ironic containing this fix.
-                if ('No conductor' in six.text_type(e) and
-                        attempt < last_attempt):
+                if 'No conductor' in str(e) and attempt < last_attempt:
                     LOG.warning('No ironic conductor is running; '
                                 'waiting...')
                     time.sleep(10)
@@ -1569,7 +1516,7 @@ class IronicDriver(virt_driver.ComputeDriver):
                   {'uuid': instance.uuid,
                    'network_info': network_info_str})
         for vif in network_info:
-            port_id = six.text_type(vif['id'])
+            port_id = str(vif['id'])
             self._plug_vif(node, port_id)
 
     def _unplug_vifs(self, node, instance, network_info):
@@ -1583,7 +1530,7 @@ class IronicDriver(virt_driver.ComputeDriver):
         if not network_info:
             return
         for vif in network_info:
-            port_id = six.text_type(vif['id'])
+            port_id = str(vif['id'])
             try:
                 self.ironicclient.call("node.vif_detach", node.uuid,
                                        port_id)
@@ -1655,7 +1602,7 @@ class IronicDriver(virt_driver.ComputeDriver):
                 admin_password, allocations, bdms, detach_block_devices,
                 attach_block_devices, network_info=None,
                 evacuate=False, block_device_info=None,
-                preserve_ephemeral=False):
+                preserve_ephemeral=False, accel_uuids=None):
         """Rebuild/redeploy an instance.
 
         This version of rebuild() allows for supporting the option to
@@ -1695,7 +1642,7 @@ class IronicDriver(virt_driver.ComputeDriver):
             information. Ignored by this driver.
         :param preserve_ephemeral: Boolean value; if True the ephemeral
             must be preserved on rebuild.
-
+        :param accel_uuids: Accelerator UUIDs. Ignored by this driver.
         """
         LOG.debug('Rebuild called for instance', instance=instance)
 
@@ -1721,8 +1668,7 @@ class IronicDriver(virt_driver.ComputeDriver):
                     files=injected_files)
             except Exception as e:
                 with excutils.save_and_reraise_exception():
-                    msg = ("Failed to build configdrive: %s" %
-                           six.text_type(e))
+                    msg = "Failed to build configdrive: %s" % str(e)
                     LOG.error(msg, instance=instance)
                     raise exception.InstanceDeployFailure(msg)
 
@@ -1740,7 +1686,7 @@ class IronicDriver(virt_driver.ComputeDriver):
                 ironic.exc.BadRequest) as e:     # Maintenance
             msg = (_("Failed to request Ironic to rebuild instance "
                      "%(inst)s: %(reason)s") % {'inst': instance.uuid,
-                                                'reason': six.text_type(e)})
+                                                'reason': str(e)})
             raise exception.InstanceDeployFailure(msg)
 
         # Although the target provision state is REBUILD, it will actually go
@@ -1969,7 +1915,7 @@ class IronicDriver(virt_driver.ComputeDriver):
             LOG.warning('Error detaching VIF from node %(node)s '
                         'after deploy failed; %(reason)s',
                         {'node': instance.node,
-                         'reason': six.text_type(e)},
+                         'reason': str(e)},
                         instance=instance)
 
     def get_volume_connector(self, instance):
@@ -2106,7 +2052,7 @@ class IronicDriver(virt_driver.ComputeDriver):
                     version=max_version)
 
     def rescue(self, context, instance, network_info, image_meta,
-               rescue_password):
+               rescue_password, block_device_info):
         """Rescue the specified instance.
 
         :param nova.context.RequestContext context:
@@ -2119,6 +2065,8 @@ class IronicDriver(virt_driver.ComputeDriver):
         :param nova.objects.ImageMeta image_meta:
             The metadata of the image of the instance. Ignored by this driver.
         :param rescue_password: new root password to set for rescue.
+        :param dict block_device_info:
+            The block device mapping of the instance.
         :raise InstanceRescueFailure if rescue fails.
         """
         LOG.debug('Rescue called for instance', instance=instance)
@@ -2129,7 +2077,7 @@ class IronicDriver(virt_driver.ComputeDriver):
             try:
                 node = self._validate_instance_and_node(instance)
             except exception.InstanceNotFound as e:
-                raise exception.InstanceRescueFailure(reason=six.text_type(e))
+                raise exception.InstanceRescueFailure(reason=str(e))
 
             if node.provision_state == ironic_states.RESCUE:
                 raise loopingcall.LoopingCallDone()
@@ -2143,20 +2091,22 @@ class IronicDriver(virt_driver.ComputeDriver):
                                    node_uuid, ironic_states.RESCUE,
                                    rescue_password=rescue_password)
         except Exception as e:
-            raise exception.InstanceRescueFailure(reason=six.text_type(e))
+            raise exception.InstanceRescueFailure(reason=str(e))
 
         timer = loopingcall.FixedIntervalLoopingCall(_wait_for_rescue)
         timer.start(interval=CONF.ironic.api_retry_interval).wait()
         LOG.info('Successfully rescued Ironic node %(node)s',
                  {'node': node_uuid}, instance=instance)
 
-    def unrescue(self, instance, network_info):
+    def unrescue(
+        self,
+        context: nova_context.RequestContext,
+        instance: 'objects.Instance',
+    ):
         """Unrescue the specified instance.
 
+        :param context: security context
         :param instance: nova.objects.instance.Instance
-        :param nova.network.model.NetworkInfo network_info:
-            Necessary network information for the unrescue. Ignored by this
-            driver.
         """
         LOG.debug('Unrescue called for instance', instance=instance)
 
@@ -2166,8 +2116,7 @@ class IronicDriver(virt_driver.ComputeDriver):
             try:
                 node = self._validate_instance_and_node(instance)
             except exception.InstanceNotFound as e:
-                raise exception.InstanceUnRescueFailure(
-                          reason=six.text_type(e))
+                raise exception.InstanceUnRescueFailure(reason=str(e))
 
             if node.provision_state == ironic_states.ACTIVE:
                 raise loopingcall.LoopingCallDone()
@@ -2180,7 +2129,7 @@ class IronicDriver(virt_driver.ComputeDriver):
             self.ironicclient.call("node.set_provision_state",
                                    node_uuid, ironic_states.UNRESCUE)
         except Exception as e:
-            raise exception.InstanceUnRescueFailure(reason=six.text_type(e))
+            raise exception.InstanceUnRescueFailure(reason=str(e))
 
         timer = loopingcall.FixedIntervalLoopingCall(_wait_for_unrescue)
         timer.start(interval=CONF.ironic.api_retry_interval).wait()
